﻿-- Author      : Sidrus
-- Create Date : 7/3/2009 12:30:41 AM
App = LibStub("AceAddon-3.0"):NewAddon("Loan Officer", "AceConsole-3.0", "AceEvent-3.0");

App.UIFrames = {
    "MainFrame",
    "IssueLoanFrame",
    "PayLoanFrame",
    "TransactionLogFrame"
};

function App:OnInitialize()
    -- register for commands and events
	self:RegisterChatCommand("loanofficer", LO_CommandHandler);
	self:RegisterChatCommand("loans", LO_CommandHandler);
	self:RegisterEvent("UNIT_TARGET");
	
	-- data members
	self.version = "0.1-beta";
	self.db = LibStub("AceDB-3.0"):New("LoanOfficer");
	self.currentTarget = "";
	self.IsDebug = false;
	
	self:VerifyDatabase();
	self:InitializeWindows();
	
	if (self.IsDebug) then
	    LO_ToggleFrame("MainFrame");
	end
	
	self:Print("Loan Officer loaded.  Use /loans or /loanofficer to open the interface.");
end
function App:InitializeWindows()
    -- Make sure all of the child windows are hidden on 
    -- start up
    for i=1,#App.UIFrames do
        LO_HideFrame(App.UIFrames[i]);
    end
end

-- CLASS PROPERTIES
function App:GetTarget()
    return self.currentTarget;
end

-- EVENT HANDLERS
function App:UNIT_TARGET()
    self.currentTarget = UnitName("target");
end

-- DATABASE FUNCTIONS
function App:EnsureAccountExists(account)
    account = string.lower(account);
    if (self.db.factionrealm[account] == nil) then
        App:PrintDebug("Creating new account for "..account);
        self.db.factionrealm[account] = {};
        self.db.factionrealm[account].transactions = {};
    end
end
function App:GetAccount(account)
    account = string.lower(account);
    return self.db.factionrealm[account];
end
function App:GetAccountList()
	local accts = {};
	local exceptions = {version=true};
	for k,v in pairs(self.db.factionrealm) do
		-- add the property as long as there are no hits for the 
		-- key in the exceptions list
		if (exceptions[k] == nil) then
			table.insert(accts,k);
		end
	end
	return accts;
end
function App:GetTransactions(accountName)
    accountName = string.lower(accountName);
    local account = self:GetAccount(accountName);
    
    -- if there's no account, there are no transactions
    if (account == nil) then
        return {};
    end
    
    return account.transactions;
end

function App:CreateLoan(loanData)
    self:RecordTransaction(loanData);
    self:Print(string.format("Created a loan for %s in the amount of: %s", loanData.borrower, LO_FormatGold(loanData.amount)));
    App:PrintBalance(loanData.borrower);
end
function App:ReceivePayment(paymentInfo)
    self:RecordTransaction(paymentInfo);
    self:Print(string.format("Received a payment for %s in the amount of: %s", paymentInfo.borrower, LO_FormatGold(paymentInfo.amount)));
    App:PrintBalance(paymentInfo.borrower);
end
function App:PrintBalance(borrower)
	local bal = App:CalculateBalance(borrower);
	self:Print(string.format("Current balance on account [%s]: %s", borrower, LO_FormatParsedGold(bal)));
end
function App:CalculateBalance(borrower)    
    -- fetch the transaction log
    local transLog = self:GetTransactions(borrower);
    App:PrintDebug("Found "..#transLog.." transactions in "..borrower.."'s account.");
    
    -- iterate over the log and tally the balance
    local balance = 0;
    for i=1,#transLog do
        local trans = transLog[i];
        balance = balance + trans.amount;
    end
    
    return LO_ParseGold(balance);
end
function App:RecordTransaction(transData)
    self:EnsureAccountExists(transData.borrower);
    local transLog = App:GetTransactions(transData.borrower);
    table.insert(transLog, {
        ["date"]   = transData.date,
        ["amount"] = transData.amount,
        ["notes"]  = transData.notes
    });
end

function App:VerifyDatabase()
    local dbVersion = (self.db.factionrealm.version or "");
    if (dbVersion == nil or (not LODBConverter:IsCurrentVersion(dbVersion))) then
        self:PrintDebug("Database version before upgrade: "..dbVersion);
        LODBConverter:UpgradeDatabase(self.db);
        local newVersion = self.db.factionrealm.version;
        self:PrintDebug("Database version after upgrade: "..newVersion);
        self:Print("Your database was at version "..dbVersion.." (out of date) and has been upgraded to the current version: "..newVersion..".");
    end
end

-- DEBUG
function App:PrintDebug(msg)
    if (App.IsDebug) then
        App:Print(msg);
    end
end

-- HELPER FUNCTIONS
function LO_CommandHandler(args)
    if (args == "") then
        LO_ToggleFrame("MainFrame");
        
    elseif (args == "show") then    
        LO_ShowFrame("MainFrame");
        
    elseif (args == "hide") then
        LO_HideFrame("MainFrame");
        
    elseif (args == "reset-db") then
        App.db:ResetDB("Default");
        App:Print("Database reset.");
	
	elseif (args == "list") then
		local accts = App:GetAccountList();
		for i=1,#accts do
			App:Print(accts[i]);
		end
			
	else
		App:Print(string.format("Unknown command: %s", tostring(args)));
		App:PrintDebug(argList);
    end
end
function LO_ToggleFrame(frame)
    -- locate the frame in the global
    local f = _G[frame];
    
    -- if the frame doesn't exist, then quit
    if (f == nil) then
        App:PrintDebug("Attempt to toggle non-existant frame: "..frame);
        return;
    end
    
    if (f:IsVisible()) then
        f:Hide();
    else
        f:Show();
    end
end
function LO_ShowFrame(frame)
    -- locate the frame in the global
    local f = _G[frame];
    
    -- if the frame doesn't exist, then quit
    if ( f==nil) then        
        App:PrintDebug("Attempt to show non-existant frame: "..frame);
        return;
    end
    
    if (not f:IsVisible()) then
        _G[frame]:Show();
    end
end
function LO_HideFrame(frame)
    -- locate the frame in the global
    local f = _G[frame];
    
    -- if the frame doesn't exist, then quit
    if ( f==nil) then
        App:PrintDebug("Attempt to hide non-existant frame: "..frame);
        return;
    end
    
    if (f:IsVisible()) then
        _G[frame]:Hide();
    end
end
function LO_ParseGold(amount)
    -- calculate the gold and fractional silver components
    local g,sTmp = math.modf(amount/10000);
    
    -- move the decimal to get the decimal silver
    sTmp = sTmp * 100;
    
    -- since the silver and copper parts of this parse shouldn't
    -- be negative (only the gold portion should be), make sure
    -- we flip the sign
    if (sTmp < 0) then 
		sTmp = -sTmp; 
	end
	
	-- calculate the silver and fractional copper components
    local s,c = math.modf(sTmp);
    
    -- move the decimal to get copper value
    c = (c * 100);
    
    return g,s,c;
end
function LO_FormatGold(amount)
    -- This function always returns a positive amount
    -- for display purposes.
    if (amount < 0) then amount = amount * -1 end  
    return string.format("%dg %ds %dc", LO_ParseGold(amount));
end
function LO_FormatParsedGold(g,s,c)
    -- This function always returns a positive amount
    -- for display purposes.
    local gold   = g or 0;
    local silver = s or 0;
    local copper = c or 0;    
    if (gold < 0) then gold = gold * -1 end    
    return string.format("%dg %ds %dc", gold, silver, copper);
end